#!/usr/bin/perl -w

#$Header: /home2/cvsroot/LogTrend/StorageServer/PostgreSQLDataAlarms.pm,v 1.15 2002/02/01 17:36:53 lsimonneau Exp $
##******************************************************************************
## Class PostgreSQLDataAlarms  isa  DataAlarms
##  Description  : class for sending data to PostgreSQL
##  Project      : LogTrend 1.0.0.0 - Atrid Systemes
##  Author       : Laurent Simonneau l.simonneau@atrid.fr
##  Author       : Sylvain Lhullier s.lhullier@atrid.fr
##  Author       : David Mascle d.mascle@atrid.fr
##******************************************************************************
#$Log: PostgreSQLDataAlarms.pm,v $
#Revision 1.15  2002/02/01 17:36:53  lsimonneau
#Minor bugfixe
#
#Revision 1.14  2002/02/01 17:35:16  lsimonneau
#Modify database structure. *_values tables inherite of data.
#
#Revision 1.13  2001/11/19 10:41:02  lsimonneau
#Critical bugfixes.
#
#Revision 1.12  2001/11/16 10:25:18  lsimonneau
#Major feature enhancement : Implementation of authentification with RSA signature.
#
#IMPORTANT : This StorageServer is incompatible with older version.
#
#Revision 1.11  2001/10/05 11:55:08  slhullier
#in the other case -> otherwise
#
#Revision 1.10  2001/06/20 13:54:25  slhullier
#
#Correction du bug GMT/LocalTime
#
#Revision 1.9  2001/06/07 14:25:40  slhullier
#
#Passage de  unshift @INC,'..';  aux packages Logtrend::....
#
#Revision 1.8  2001/03/12 13:35:37  slhullier
#
#Ajout du champ lastconnectiondate dans la table agents
#
#Revision 1.7  2001/03/07 15:40:32  slhullier
#
#Verification qu'une alarme n'est pas deja presente et non acquittee
#
#Revision 1.6  2001/03/07 15:15:30  slhullier
#
#LogDie mis en place sur le serveur et l'agent Linux
#
#Revision 1.5  2001/03/02 09:30:50  slhullier
#
#Suppression de la table SOURCES_AGENTS
#
#Revision 1.4  2001/03/01 17:21:37  slhullier
#
#Commentaires
#
#Revision 1.3  2001/03/01 16:40:37  slhullier
#
#Fautes
#
#Revision 1.2  2001/03/01 16:28:50  slhullier
#
#Premier parcours des fichiers
#
#Revision 1.1.1.1  2001/02/06 09:48:28  fdubuy
#
#First CVS version : all work done by D.Mascle
#
#

use strict;
use DBI;

package LogTrend::StorageServer::PostgreSQLDataAlarms;
use LogTrend::Common::LogDie;
use LogTrend::StorageServer::DataAlarms;
use LogTrend::StorageServer::ErrorDeclaration;

use vars qw( @ISA );
@ISA = ("LogTrend::StorageServer::DataAlarms");

##******************************************************************************
## Constructor  public  (>DataAlarms)
##  Parameters   : the name of the database, the name or the IP of the host of
##                 the database server, the port on which the database server
##                 is listening for connections, the name of a database user,
##                 the password of a database user
##
##  Throw errors : Error::DB_Connection
##******************************************************************************
sub new
{
   my ($classname, $databasename, $host, $port, $username, $password) = @_;
   my $self = $classname->SUPER::new($databasename, $host, $port, $username, $password);
   bless($self, $classname);
   return $self;
}

##*****************************************************************************
## connect protected
##  Description  : Try to establish a connection with the database
##  Parameters   : none
##*****************************************************************************
sub connect
{
    my ($self, $databasename, $host, $port, $username, $password) = @_;

    die("Already connected") if defined $self->{DBHANDLE};

    my $ConnectionParameters = "dbi:Pg(AutoCommit=>0):dbname=$databasename";
    
    # Build the connection parametres
    # dbi:Pg:dbname=logtrend;host=serveur.orsay.atrid.fr;port=5432
    $ConnectionParameters = $ConnectionParameters.";host=".$host;
    
    $ConnectionParameters = $ConnectionParameters.";port=".$port;
    
    # Connect to the DB
    my $db_handle = DBI->connect($ConnectionParameters, $username, $password, {PrintError => 0});
    
    my $result = 0;
    if (!defined $db_handle) {
        throw Error::DB_Connection($DBI::errstr);
    }

    $self->{DBHANDLE} = $db_handle;
}

sub commit {
    my $self = shift;

    $self->{DBHANDLE}->commit;
}

sub rollback {
    my $self = shift;

    $self->{DBHANDLE}->rollback;
}


##*****************************************************************************
## disconnect protected
##  Description  : Disconnect from the database
##  Parameters   : none
##*****************************************************************************
sub disconnect
{
    my $self = shift;
    
    if(defined $self->{DBHANDLE}) {
        $self->{DBHANDLE}->disconnect()
            or throw Error::DB_Connection($self->{DBHANDLE}->errstr);
    }
}

##******************************************************************************
## Method UpdateLastConnectionDate  private
##  Description  : gets an alarm's description id
##  Parameters   : the agent's source, the agent's number, the agent's version,
##                 the agent's activation date
##
##  Throw error  : Error::DB_Request
##******************************************************************************
sub UpdateLastConnectionDate
{
   my ($self, $agentsource, $agentnumber, $agentversion, $activationdate) = @_;
   my $result = 0;
   my $st_handle;
   my $date = gmtime(time())." GMT";

   $st_handle = $self->{DBHANDLE}->prepare("

UPDATE agents 

SET lastconnectiondate = ? 

WHERE (id_source = ?) AND 
      (number = ?) AND 
      (version = ?) AND 
      (activationdate = ?)

") or throw Error::DB_Request($self->{DBHANDLE}->errstr);

   $st_handle->execute($date, $agentsource, $agentnumber, $agentversion, $activationdate)
       or throw Error::DB_Request($st_handle->errstr);
}


##   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
##  @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
## @@@
## @@                  ALARMS SKILLS
## @@@
##  @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
##   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

##******************************************************************************
## Function AddAlarm  public  (>DataAlarms)
##  Description  : adds an alarm to the database
##  Parameters   : the agent's source, the agent's number, the agent's version,
##                 the agent's activation date, the alarm's collection date,
##                 the alarm's description number
##
##  Throw error  : Error::DB_Request
##                 Error::DB_AgentDescription (from GetIdAlarmDescription)
##******************************************************************************
sub AddAlarm
{
   my ($self, $agentsource, $agentnumber, $agentversion, $activationdate,
       $collectiondate, $number) = @_;
   my $st_handle;

   # Does the AlarmDescription exit ? (don't catch error)
   my $id_alarm_description = $self->GetIdAlarmDescription($agentsource,
                        $agentnumber, $agentversion, $activationdate, $number);

   # Does the Alarm already exists at the sames dates ?
   my $storagedate = gmtime(time())." GMT";

   # Does the Alarm already exists without acquittal ?
   return if $self->CheckIfAlarmAlreadyExistsWithoutAcquittal($id_alarm_description);

   # we add the alarm
   $st_handle = $self->{DBHANDLE}->prepare("

INSERT INTO alarms(id_alarm_description, storage_date, release_date) 

VALUES (?, ?, ?)

") or throw Error::DB_Request($self->{DBHANDLE}->errstr);

   $st_handle->execute($id_alarm_description, $storagedate, $collectiondate)
       or throw Error::DB_Request($st_handle->errstr);       
}

##******************************************************************************
## Method CheckIfAlarmAlreadyExistsWithoutAcquittal  private
##  Description  : checks if an alarm already exists without acquittal
##  Parameters   : the alarm's description id
##  Return value : 1 if succesfull and 0 otherwise
##
##  Throw error  : Error::DB_Request
##******************************************************************************
sub CheckIfAlarmAlreadyExistsWithoutAcquittal
{
   my ($self, $id_alarm_description) = @_;
   my $st_handle;
   my @row;

   $st_handle = $self->{DBHANDLE}->prepare("

SELECT COUNT(*) 

FROM alarms 

WHERE (id_alarm_description = ?) AND 
      (acquittal_date IS NULL)

") or throw Error::DB_Request($self->{DBHANDLE}->errstr);       

   $st_handle->execute($id_alarm_description)
       or throw Error::DB_Request($st_handle->errstr);

   @row = $st_handle->fetchrow_array();
   
   if(defined $st_handle->err) {
       throw Error::DB_Request($st_handle->errstr);
   }

   return $row[0];
}

##******************************************************************************
## Method GetIdAlarmDescription  private
##  Description  : gets an alarm's description id
##  Parameters   : the agent's source, the agent's number, the agent's version,
##                 the agent's activation date, the alarm's description number
##  Return value : an id if successfull and 0 otherwise
##
##  Throw error  : Error::DB_Request
##                 Error::DB_Internal
##                 Error::DB_AgentDescription
##******************************************************************************
sub GetIdAlarmDescription
{
   my ($self, $agentsource, $agentnumber, $agentversion, $activationdate, $number) = @_;
   my $st_handle;
   my @row;

   $st_handle = $self->{DBHANDLE}->prepare("

SELECT aad.id_alarm_description 

FROM agents AS a, 
     agents_alarms_descriptions AS aad 

WHERE (a.id_source = ?) AND (a.number = ?) AND (a.version = ?) AND 
      (a.activationdate = ?) AND (a.id_agent = aad.id_agent) AND (aad.alarm_number = ?)

") or throw Error::DB_Request($self->{DBHANDLE}->errstr);       
  
   $st_handle->execute($agentsource, $agentnumber, $agentversion, $activationdate, $number)
       or throw Error::DB_Request($st_handle->errstr);
   
   @row = $st_handle->fetchrow_array();

   if(defined $st_handle->err) {
       throw Error::DB_Request($st_handle->errstr);
   }

   if(! @row) {
       throw Error::DB_AgentDescription("Can't find alarm $number for agent $agentnumber on source $agentsource.");
   }

   return $row[0];
}

##   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
##  @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
## @@@
## @@                  DATA SKILLS
## @@@
##  @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
##   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

##******************************************************************************
## Function AddData  public (>DataAlarms)
##  Description  : adds a data to the database
##  Parameters   : the agent's source, the agent's number, the agent's version,
##                 the agent's activation date, the data's collection date,
##                 the data's description number, the data's value
##  Return value : 1 if successfull and 0 otherwise
##
##  Throw error  : Error::DB_Request
##                 Error::DB_AgentDescription (from GetIdDataDescription)
##******************************************************************************
sub AddData
{
   my ($self, $agentsource, $agentnumber, $agentversion, $activationdate,
       $collectiondate, $number, $value) = @_;

   ## Dont catch error (throw Error::DB_AgentDescription)
   my $id_data_description = $self->GetIdDataDescription($agentsource, $agentnumber, 
							 $agentversion, $activationdate, 
							 $number);

   my $storagedate = gmtime(time())." GMT";

   $self->AddDataAndValue($id_data_description, $storagedate, $collectiondate, $value);
}

##******************************************************************************
## Method GetIdDataDescription  private
##  Description  : gets a data's description id
##  Parameters   : the agent's source, the agent's number, the agent's version,
##                 the agent's activation date, the data's description number
##  Return value : an id 
##
##  Throw error  : Error::DB_Request
##******************************************************************************
sub GetIdDataDescription
{
   my ($self, $agentsource, $agentnumber, $agentversion, $activationdate, $number) = @_;
   my $result = 0;
   my $st_handle;
   my @row;

   $st_handle = $self->{DBHANDLE}->prepare("

SELECT adds.id_data_description 

FROM agents AS a, agents_data_descriptions AS adds 

WHERE (a.id_source = ?) AND (a.number = ?) AND (a.version = ?) AND 
      (a.activationdate = ?) AND (a.id_agent = adds.id_agent) AND (adds.number = ?)

") or throw Error::DB_Request($self->{DBHANDLE}->errstr);

   $st_handle->execute($agentsource, $agentnumber, $agentversion, $activationdate, $number)
       or throw Error::DB_Request($st_handle->errstr);

   @row = $st_handle->fetchrow_array();

   if(defined $st_handle->err) {
       throw Error::DB_Request($st_handle->errstr);
   }
 
   if(! @row) {
       throw Error::DB_AgentDescription("Can't find data $number for agent $agentnumber on source $agentsource.");
   }

   return $row[0];
}

##******************************************************************************
## Method AddDataAndValue  private
##  Description  : adds a data and its value in the database
##  Parameters   : a data's description id, the data's storage date,
##                 the data's collection date, the data's value
##
##  Throw error  : Error::DB_Request
##                 Error::DB_AgentDescription
##******************************************************************************
sub AddDataAndValue
{
   my ($self, $id_data_description, $storagedate, $collectiondate, $value) = @_;
   my $result = 0;

   my $datatype = $self->GetDataType($id_data_description);
   
   my $st_handle = $self->{DBHANDLE}->prepare("

INSERT INTO ${datatype}_values(id_data_description, collection_date, storage_date, value)

VALUES (?, ?, ?, ?)

") or throw Error::DB_Request($self->{DBHANDLE}->errstr);
   
   $st_handle->execute($id_data_description, $collectiondate, $storagedate, $value)
       or throw Error::DB_Request($st_handle->errstr);
}



##******************************************************************************
## Method GetDataType  private
##  Description  : gets a data's type
##  Parameters   : a data's description id
##  Return value : a type if successfull and undef otherwise
##
##  Throw error  : Error::DB_Request
##                 Error::DB_AgentDescription
##******************************************************************************
sub GetDataType
{
   my ($self, $id_data_description) = @_;
   my $st_handle;
   my @row;

   $st_handle = $self->{DBHANDLE}->prepare("

SELECT dt.type 

FROM agents_data_descriptions AS adds, 
     data_types AS dt 

WHERE (adds.id_data_description = ?) AND 
      (adds.type = dt.id_data_type)

") or throw Error::DB_Request($self->{DBHANDLE}->errstr);
   
   $st_handle->execute($id_data_description)
       or throw Error::DB_Request($st_handle->errstr);
  
   @row = $st_handle->fetchrow_array();

   if(defined $st_handle->err) {
       throw Error::DB_Request($st_handle->errstr);
   }

   if (! $row[0]) {
       throw Error::DB_AgentDescription("Can't find data type for data description $id_data_description.");
   }  

   return $row[0];
}

##******************************************************************************
1;
