#!/usr/bin/perl -w

#$Header: /home2/cvsroot/LogTrend/StorageServer/StorageProtocol.pm,v 1.13 2001/12/10 17:37:28 lsimonneau Exp $
##******************************************************************************
## Class StorageProtocol
##  Description  : Manage protocol in XML
##  Project      : LogTrend 1.0.0.0 - Atrid Systemes
##  Author       : Laurent Simonneau l.simonneau@atrid.fr
##  Author       : Sylvain Lhullier s.lhullier@atrid.fr
##  Author       : David Mascle d.mascle@atrid.fr
##******************************************************************************
#$Log: StorageProtocol.pm,v $
#Revision 1.13  2001/12/10 17:37:28  lsimonneau
#Minor bugfixes.
#
#Revision 1.12  2001/11/16 10:25:18  lsimonneau
#Major feature enhancement : Implementation of authentification with RSA signature.
#
#IMPORTANT : This StorageServer is incompatible with older version.
#
#Revision 1.11  2001/10/05 11:55:08  slhullier
#in the other case -> otherwise
#
#Revision 1.10  2001/06/07 14:25:40  slhullier
#
#Passage de  unshift @INC,'..';  aux packages Logtrend::....
#
#Revision 1.9  2001/03/22 13:32:24  slhullier
#
#Petit bug du serveur du au parseur
#
#Revision 1.8  2001/03/12 13:35:37  slhullier
#
#Ajout du champ lastconnectiondate dans la table agents
#
#Revision 1.7  2001/03/07 15:40:32  slhullier
#
#Verification qu'une alarme n'est pas deja presente et non acquittee
#
#Revision 1.6  2001/03/07 15:15:30  slhullier
#
#LogDie mis en place sur le serveur et l'agent Linux
#
#Revision 1.5  2001/03/06 13:25:43  slhullier
#
#Plus d'explications
#
#Revision 1.4  2001/03/05 14:43:09  slhullier
#
#Bug alarm (")
#
#Revision 1.3  2001/03/01 17:21:37  slhullier
#
#Commentaires
#
#Revision 1.2  2001/03/01 16:28:50  slhullier
#
#Premier parcours des fichiers
#
#Revision 1.1.1.1  2001/02/06 09:48:28  fdubuy
#
#First CVS version : all work done by D.Mascle
#
#

package LogTrend::StorageServer::StorageProtocol;

use strict;
use XML::DOM;
use Error qw(:try);
use LogTrend::Common::LogDie;
use LogTrend::StorageServer::ErrorDeclaration;

##   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
##  @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
## @@@
## @@                  PUBLIC METHODS
## @@@
##  @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
##   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


##******************************************************************************
## Constructor  public
##  Parameters   : an authentication interface, a storage interface
##******************************************************************************
sub new
{
   my ($classname, $authenticationinterface, $storageinterface) = @_;
   my $self = {};
   bless($self, $classname);
   $self->{AUTHENTICATIONINTERFACE} = $authenticationinterface;
   $self->{STORAGEINTERFACE} = $storageinterface;
   return $self;
}

##******************************************************************************
## Method ProcessInput  public
##  Description  : process a message from an agent and builds a response
##  Parameters   : a message
##
##  Throw errors : Error::SP_Internal
##                 Error::SP_Protocol
##                 Error::SP_XML
##
##  Don't catch error  : Error::SP_XML
##                       Error::DB_Request
##                       Error::DB_AgentDescription
##                       Error::Not_A_Signature (from LogTrend::Crypto::KeyManager)
##                       Error::Authentication  (from LogTrend::Crypto::KeyManager)
##                       Error::IO  (from LogTrend::Crypto::KeyManager)
##******************************************************************************
sub ProcessInput
{
   my ($self, $message) = @_;
   my $command = undef;
   my $parameters = undef;

   ($command, $parameters) = split /[ \n]/, $message, 2;

   chomp($parameters);

   if (! defined $command) {
       throw Error::SP_Internal("Undefined variable \$command");
   }

   if ($command eq "DATA") {
       if(!defined $parameters) {
	   throw Error::SP_Protocol("Usage :\n\tDATA\n\t<ADAT>\n\t  ...\n\t</ADAT>\n\t.");
       }

       $self->{DATA} = $parameters;
   }

   elsif ($command eq "SIGNATURE") {
       if(!defined $parameters) {
	   throw Error::SP_Protocol("Usage :\n\tSIGNATURE\n".
				    "\t-----BEGIN PGP SIGNATURE-----\n\n".
				    "\t <PGP signature armoured data>\n".
				    "\t-----END PGP SIGNATURE-----\n\n\t.\n");
       }       

       $self->{SIGNATURE} = $parameters;
   }
   
   
   elsif ($command eq "QUIT") {
       if(!defined $self->{DATA} and !defined $self->{SIGNATURE}) {
	   return 1;
       }
       elsif(!defined $self->{DATA} or !defined $self->{SIGNATURE}) {
	   throw Error::SP_Protocol("Usage :\n".
				    "\tDATA\n".
				    "\t<xml data>\n".
				    "\t.\n".
				    "\tSIGNATURE\n".
				    "\t<rsa signature data>\n".
				    "\t.\n");
       }
       
       my $verify = $self->{AUTHENTICATIONINTERFACE}->verify_signature($self->{DATA}, $self->{SIGNATURE});
       
       $self->{AUTHENTICATED_SOURCE} = $verify->{source_id};

       try {
           $self->TreatDataAndAlarms;
       }

       ## Rollback if an error occure
       otherwise {
           my $error = shift;
           $self->{STORAGEINTERFACE}->rollback;
           $error->throw;
       }
       
       ## If evrything is OK, commit changes in DB
       finally {
           $self->{STORAGEINTERFACE}->commit;
       };
       
       return 1;
   }
   
   elsif ($command eq "CANCEL") {
       undef $self->{DATA};
       undef $self->{SIGNATURE};
   }

   else {
       throw Error::SP_Protocol("Unknown command $command");
   }

   return 0;
}


##   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
##  @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
## @@@
## @@                  PRIVATE METHODS
## @@@
##  @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
##   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


##******************************************************************************
## Method TreatDataAndAlarms  private
##  Description  : analyze XML data and alarms of an agent and add them to the database
##  Parameters   : a XML data and alarms message
##
##  Throw error  : Error::SP_XML
##
##  Don't catch error  : Error::SP_XML
##                       Error::DB_Request
##                       Error::DB_AgentDescription
##******************************************************************************
sub TreatDataAndAlarms
{
   my $self = shift;
   my $xml_data = $self->{DATA};

   # creates the DOM
   my $parser = new XML::DOM::Parser(ErrorContext => 1);
   # parses the agent's description file
   my $doc = undef;

   eval
   {
       # the eval is used to prevent from savage exits from the DOM object
       $doc = $parser->parse($xml_data);
   };
   
   if($@) {
       throw Error::SP_XML($@);
   }
   
   # get the root node of the document
   my @rootnodelist = $doc->getElementsByTagName("ADAT")
       or throw Error::SP_XML($!);
   
   if (@rootnodelist == 0)
   {
       throw Error::SP_XML("Can't find ADAT tag in the xml data");
   }
   
   my $rootnode = $rootnodelist[0];
       
   my @agentdataalarmsnodelist = $rootnode->getElementsByTagName("ADA");
   if (@agentdataalarmsnodelist == 0)
   {
       throw Error::SP_XML("Can't find ADA tag(s) in the xml data");
   }
   
   my $nbagentdataalarmsnode = @agentdataalarmsnodelist;
   for (my $i = 0; $i < $nbagentdataalarmsnode; $i++)
   {
       my $agentdataalarmsnode = $agentdataalarmsnodelist[$i];
       
       ## Don't catch errors
       $self->{SOURCE} = $self->GetAgentSource($agentdataalarmsnode);

       if($self->{SOURCE} != $self->{AUTHENTICATED_SOURCE}) {
           throw Error::Permission("You are authenticated as source number $self->{AUTHENTICATED_SOURCE}\n".
                                   "You can't access to data of source $self->{SOURCE}\n");
       }

       $self->{NUMBER} = $self->GetAgentNumber($agentdataalarmsnode);
       $self->{VERSION} = $self->GetAgentVersion($agentdataalarmsnode);
       $self->{ACTIVATIONDATE} = $self->GetAgentActivationDate($agentdataalarmsnode);
       $self->{COLLECTIONDATE} = $self->GetCollectionDate($agentdataalarmsnode);
       
       my $datanode = $self->GetDataNode($agentdataalarmsnode);
       my $alarmsnode = $self->GetAlarmsNode($agentdataalarmsnode);
	   
       $self->AddData($datanode);
       $self->AddAlarms($alarmsnode);
       $self->{STORAGEINTERFACE}->UpdateLastConnectionDate($self->{SOURCE}, $self->{NUMBER},
							   $self->{VERSION}, 
							   $self->{ACTIVATIONDATE});
   }
}

##******************************************************************************
## Method GetANodeRef  private
##  Description  : gets the first subnode reference of a node
##  Parameters   : a node and a subnode name
##  Return value : a subnode reference
##
##  Throw error  : Error::SP_XML
##******************************************************************************
sub GetANodeRef
{
   my ($self, $node, $nodename) = @_;
   my $result = undef;

   my $nodelist = $node->getElementsByTagName($nodename);

   if(@$nodelist == 0) {
       throw Error::SP_XML("Can't find $nodename tag");
   }

   $result = $nodelist->item(0); # we suppose that it's the good one

   return $result;
}

##******************************************************************************
## Method GetANode  private
##  Description  : gets the first subnode of a node
##  Parameters   : a node and a subnode name
##  Return value : a subnode
##
##  Throw error  : Error::SP_XML
##******************************************************************************
sub GetANode
{
   my ($self, $node, $nodename) = @_;
   my $result = undef;

   my $nodelist = $node->getElementsByTagName($nodename);

   if(@$nodelist == 0) {
       throw Error::SP_XML("Can't find $nodename tag");
   }

   my $thenode = $nodelist->item(0);

   $result = $self->GetValue($thenode);

   if(!defined $result) {
       throw Error::SP_XML("Can't find the value of tag $nodename");
   }

   return $result;
}

##******************************************************************************
## Method GetValue  private
##  Description  : gets the value associated to a node
##  Parameters   : a node
##  Return value : a node value
##
##  Don't catch error  : Error::SP_XML
##******************************************************************************
sub GetValue
{
   my ($self, $node) = @_;
   my $result = undef;

   my $nodevalueref = $node->getFirstChild();
   if(defined $nodevalueref) {
       $result = $nodevalueref->getNodeValue();
   }

   return $result;
}

##******************************************************************************
## Method GetAgentSource  private
##  Description  : gets the source of an agent
##  Parameters   : the parent's node of the source node
##  Return value : the source of an agent
##
##  Don't catch error  : Error::SP_XML
##******************************************************************************
sub GetAgentSource
{
   my ($self, $node) = @_;
   my $result;

   $result = $self->GetANode($node, "S");

   return $result;
}

##******************************************************************************
## Method GetAgentNumber  private
##  Description  : gets the number of an agent
##  Parameters   : the parent's node of the number node
##  Return value : the number of an agent
##
##  Don't catch error  : Error::SP_XML
##******************************************************************************
sub GetAgentNumber
{
   my ($self, $node) = @_;
   my $result;

   $result = $self->GetANode($node, "N");

   return $result;
}

##******************************************************************************
## Method GetAgentVersion  private
##  Description  : gets the version of an agent
##  Parameters   : the parent's node of the version node
##  Return value : the version of an agent
##
##  Don't catch error  : Error::SP_XML
##******************************************************************************
sub GetAgentVersion
{
   my ($self, $node) = @_;
   my $result;

   $result = $self->GetANode($node, "V");

   return $result;
}

##******************************************************************************
## Method GetAgentActivationDate  private
##  Description  : gets the activationdate of an agent
##  Parameters   : the parent's node of the activationdate node
##  Return value : the activationdate of an agent
##
##  Don't catch error  : Error::SP_XML
##******************************************************************************
sub GetAgentActivationDate
{
   my ($self, $node) = @_;
   my $result;

   $result = $self->GetANode($node, "AD");

   return $result;
}

##******************************************************************************
## Method GetCollectionDate  private
##  Description  : gets the collection date of the data and the alarms
##  Parameters   : the parent's node of the collectiondate node
##  Return value : the collection date of the data and the alarms
##
##  Don't catch error  : Error::SP_XML
##******************************************************************************
sub GetCollectionDate
{
   my ($self, $node) = @_;
   my $result;

   $result = $self->GetANode($node, "CD");

   return $result;
}

##******************************************************************************
## Method GetDataNode  private
##  Description  : gets an agent's data's node
##  Parameters   : the parent's node of an agent's data's node
##  Return value : the agent's data's node
##
##  Don't catch error  : Error::SP_XML
##******************************************************************************
sub GetDataNode
{
   my ($self, $node) = @_;
   my $result;

   $result = $self->GetANodeRef($node, "DT");

   return $result;
}

##******************************************************************************
## Method GetAlarmsNode  private
##  Description  : gets an agent's alarm's node
##  Parameters   : the parent's node of an alarm's data's node
##  Return value : the agent's alarm's node
##
##  Don't catch error  : Error::SP_XML
##******************************************************************************
sub GetAlarmsNode
{
   my ($self, $node) = @_;
   my $result;

   $result = $self->GetANodeRef($node, "AT");

   return $result;
}

##******************************************************************************
## Method AddData  private
##  Description  : adds all the data from a data's node list to the database
##  Parameters   : a data's node list
##
##  Don't catch error  : Error::SP_XML
##                       Error::DB_Request
##                       Error::DB_AgentDescription
##******************************************************************************
sub AddData
{
   my ($self, $node) = @_;

   my @datanodelist = $node->getElementsByTagName("D");

   my $nbdatanode = @datanodelist;
   for (my $i = 0; $i < $nbdatanode; $i++) {
       my ($datanumber, $datavalue) = $self->GetDataNumberAndValue($datanodelist[$i]);

       $self->{STORAGEINTERFACE}->AddData(  $self->{SOURCE},
					    $self->{NUMBER}, $self->{VERSION},
					    $self->{ACTIVATIONDATE}, $self->{COLLECTIONDATE},
					    $datanumber, $datavalue  );
   }
}

##******************************************************************************
## Method GetAttributeValue  private
##  Description  : gets the value of a given attribute of a given node
##  Parameters   : a node and an attribute name
##  Return value : an attribute value (a string) if successfull and undef otherwise
##
##  Throw error  : Error::SP_XML
##******************************************************************************
sub GetAttributeValue
{
   my ($self, $node, $attrname) = @_;
   my $result;

   my $attributes = $node->getAttributes();

   if (! defined $attributes)
   {
       throw Error::SP_XML("Can't find attribute $attrname");
   }
   
   my $attrnode = $attributes->getNamedItem($attrname);
   if (! defined $attrnode) {
       throw Error::SP_XML("Can't find attribute $attrname");
   }

   $result = $attrnode->getValue();

   if (! defined $result) {
       throw Error::SP_XML("Can't find value for attribute $attrname");
   }

   return $result;
}

##******************************************************************************
## Method GetDataNumberAndValue  private
##  Description  : gets the number and the value of a data
##  Parameters   : a data's node
##  Return value : data number and value
##
##  Don't catch error  : Error::SP_XML
##******************************************************************************
sub GetDataNumberAndValue
{
   my ($self, $datanode) = @_;

   my $number = $self->GetAttributeValue($datanode, "N");
   my $value = $self->GetAttributeValue($datanode, "V");

   return ($number, $value);
}

##******************************************************************************
## Method AddAlarms  private
##  Description  : adds all the alarms from an alarm's node list to the database
##  Parameters   : an alarm's node list
##
##  Don't catch error  : Error::SP_XML
##                       Error::DB_Request
##                       Error::DB_AgentDescription
##******************************************************************************
sub AddAlarms
{
   my ($self, $node) = @_;

   my @alarmsnodelist = $node->getElementsByTagName("A");
   
   my $nbalarmsnode = @alarmsnodelist;
   for (my $i = 0; $i < $nbalarmsnode; $i++) {
       my $alarmnumber = $self->GetAlarmNumber($alarmsnodelist[$i]);

       $self->{STORAGEINTERFACE}->AddAlarm(  $self->{SOURCE},
					     $self->{NUMBER}, $self->{VERSION},
					     $self->{ACTIVATIONDATE}, $self->{COLLECTIONDATE},
					     $alarmnumber  );
   }
}

##******************************************************************************
## Method GetAlarmNumber private
##  Description  : gets the alarm's number of an alarm node in the DOM object
##  Parameters   : an alarm's node
##  Return value : an alarm's number if successfull and undef otherwise
##
##  Don't catch error  : Error::SP_XML
##******************************************************************************
sub GetAlarmNumber
{
   my ($self, $alarmnode) = @_;

   my $number = $self->GetAttributeValue($alarmnode, "N");

   return $number;
}

##******************************************************************************
1;
